require "prefabutil"

assets =
{
	Asset("ANIM", "anim/firepit.zip"),
	Asset("SOUND", "sound/common.fsb"),
}

local prefabs = 
{
    "campfirefire",
	"pigtorch_fuel",
	"wanawanatiki",
	"collapse_small",
}

local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_stone")
	inst:Remove()
end

local function onhit(inst,worker)
	if inst.components.childspawner.child and inst.components.childspawner.child.components.combat then
	    inst.components.childspawner.child.components.combat:SuggestTarget(worker)
	end
	inst.AnimState:PlayAnimation("hit")
	inst.AnimState:PushAnimation("idle")
end

local function onignite(inst)
    if not inst.components.cooker then
        inst:AddComponent("cooker")
    end
end
local function onextinguish(inst)
    if inst.components.cooker then
        inst:RemoveComponent("cooker")
    end
    if inst.components.fueled then
        inst.components.fueled:InitializeFuelLevel(0)
    end
end

local function fn()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "firepit.png" )
	minimap:SetPriority( 1 )

	MakeObstaclePhysics(inst, 0.33)

	inst:AddComponent("inspectable")


	inst:AddComponent("burnable")
    inst.components.burnable:AddBurnFX("campfirefire", Vector3(0,.4,0) )
    inst.components.burnable:MakeNotWildfireStarter()
    inst:ListenForEvent("onextinguish", onextinguish)
    inst:ListenForEvent("onignite", onignite)

    inst:AddComponent("fueled")
    inst.components.fueled.accepting = true
    inst.components.fueled.maxfuel = TUNING.PIGTORCH_FUEL_MAX
    inst.components.fueled:SetSections(3)
    inst.components.fueled.fueltype = "PIGTORCH"    
    inst.components.fueled:SetUpdateFn( function()
        local rate = 1 
        if GetSeasonManager():IsRaining() then
            rate = rate + TUNING.PIGTORCH_RAIN_RATE*GetSeasonManager():GetPrecipitationRate()
        end
        rate = rate + TUNING.PIGTORCH_WIND_RATE * GetSeasonManager():GetHurricaneWindSpeed() 

        inst.components.fueled.rate = rate
        
        if inst.components.burnable and inst.components.fueled then
            inst.components.burnable:SetFXLevel(inst.components.fueled:GetCurrentSection(), inst.components.fueled:GetSectionPercent())
        end
    end)
    inst.components.fueled:SetSectionCallback( function(section)
        if section == 0 then
            inst.components.burnable:Extinguish()
        else
            if not inst.components.burnable:IsBurning() then
                inst.components.burnable:Ignite()
            end
            
            inst.components.burnable:SetFXLevel(section, inst.components.fueled:GetSectionPercent())
        end
    end)
    inst.components.fueled:InitializeFuelLevel(TUNING.PIGTORCH_FUEL_MAX)

    anim:SetBank("firepit")
    anim:SetBuild("firepit")
    anim:PlayAnimation("idle",false)


	inst:AddTag("structure")
	inst:AddComponent("lootdropper")
	inst.components.lootdropper:SetLoot({"log", "rocks", "rocks", "poop"})
	inst:AddComponent("workable")
	inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
	inst.components.workable:SetWorkLeft(8)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)
	
	inst:AddComponent( "childspawner" )
	inst.components.childspawner:SetSpawnPeriod(1)
	inst.components.childspawner:SetMaxChildren(1)
	inst.components.childspawner.childname = "wanawanatiki"
    inst.components.childspawner:StartSpawning()
	return inst
end

return Prefab("forest/objects/tikifire", fn, assets, prefabs)
