

local assets =
{
	Asset("ANIM", "anim/tiki_head.zip"),
    Asset("SOUND", "sound/monkey.fsb"),
    Asset("MINIMAP_IMAGE", "monkey_barrel"),
}

local prefabs =
{
    "monkey",
    "poop",
    "cave_banana",
    "collapse_small",
}

SetSharedLootTable( 'monkey_barrel',
{
    {'poop',        1.0},
    {'poop',        1.0},
    {'cave_banana', 1.0},
    {'cave_banana', 1.0},
    {'trinket_4',   .01},
})

local function shake(inst)
    local anim = ((math.random() > .5) and "move1") or "move2"
    inst.AnimState:PlayAnimation(anim)
    inst.AnimState:PushAnimation("idle")
    inst.SoundEmitter:PlaySound("dontstarve/creatures/monkey/barrel_rattle")

	GetPlayer().components.builder:EvaluateTechTrees()
end

local function onhammered(inst, worker)
    inst.shake:Cancel()
    inst.shake = nil
    inst.components.lootdropper:DropLoot()
    SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
    inst:Remove()
end

local function onhit(inst, worker)
    inst.AnimState:PlayAnimation("hit")
    inst.AnimState:PushAnimation("idle", false)

    inst.shake:Cancel()
    inst.shake = nil
    inst.shake = inst:DoPeriodicTask(GetRandomWithVariance(10, 3), shake)
end
-------------------------------------------------------
local function OnTurnOn(inst)
    inst.components.prototyper.on = true  -- prototyper doesn't set this until after this function is called!!
	inst:DoTaskInTime(.1, shake)

    local attacker = FindEntity(inst, 5, function(guy)
        return guy:HasTag("character") and not guy:HasTag("tiki") or guy:HasTag("animal") and not guy:HasTag("tiki")
    end)
	inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("tiki") and not dude:HasTag("player") end, 10)
end

local function OnTurnOff(inst)
    inst.components.prototyper.on = false  -- prototyper doesn't set this until after this function is called
	local playervar = GetPlayer()
	if playervar then
		playervar.components.builder.accessible_tech_trees = TECH.NONE
	end
	inst:DoTaskInTime(.1, shake)

    local attacker = FindEntity(inst, 5, function(guy)
        return guy:HasTag("character") and not guy:HasTag("tiki") or guy:HasTag("animal") and not guy:HasTag("tiki")
    end)
	inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("tiki") and not dude:HasTag("player") end, 10)
end
-------------------------------------------------------
local function fn()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    MakeObstaclePhysics( inst, 1)

    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon("tiki.tex")
	minimap:SetPriority( 5 )

    anim:SetBank("barrel")
    anim:SetBuild("tiki_head")
    anim:PlayAnimation("idle", true)

	inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('monkey_barrel')

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
    inst.components.workable:SetOnFinishCallback(onhammered)
    inst.components.workable:SetOnWorkCallback(onhit)

    inst:AddComponent("inspectable")

    inst.shake = inst:DoPeriodicTask(GetRandomWithVariance(15, 3), shake )
	
	inst:AddTag("prototyper")
	inst:AddComponent("prototyper")
	inst.components.prototyper.onturnon = OnTurnOn
	inst.components.prototyper.onturnoff = OnTurnOff
	inst.components.prototyper.trees = TUNING.PROTOTYPER_TREES.TRIBAL

	inst:AddComponent("combat")
	
	return inst
end

return Prefab( "cave/objects/tikihead", fn, assets, prefabs)







