require "brains/wanawanabrain"
require "stategraphs/SGpig"

local assets =
{
	Asset("ANIM", "anim/ds_pig_basic.zip"),
	Asset("ANIM", "anim/ds_pig_actions.zip"),
	Asset("ANIM", "anim/ds_pig_attacks.zip"),
	Asset("SOUND", "sound/pig.fsb"),
	Asset("ANIM", "anim/wanawana_build.zip"),
}

local prefabs =
{
    "meat",
    "monstermeat",
    "poop",
    "pigskin",
}

local MAX_TARGET_SHARES = 5
local SHARE_TARGET_DIST = 30

local function ontalk(inst, script)
	--inst.SoundEmitter:PlaySound("dontstarve/pig/grunt") --HES NOT A PIG!
end

local function SpringMod(amt)
    if GetSeasonManager() and (GetSeasonManager():IsSpring() or GetSeasonManager():IsGreenSeason()) then
        return amt * TUNING.SPRING_COMBAT_MOD
    else
        return amt
    end
end

local function OnEat(inst, food)    
    if food.components.edible and food.components.edible.foodtype == "VEGGIE" then
		local poo = SpawnPrefab("poop")
		poo.Transform:SetPosition(inst.Transform:GetWorldPosition())		
	end
    
end

local function OnAttacked(inst, data)
    --print(inst, "OnAttacked")
    local attacker = data.attacker
    inst:ClearBufferedAction()

	if attacker.prefab ~= "tikiman" then
        inst.components.combat:SetTarget(attacker)
		inst.components.combat:ShareTarget(attacker, SHARE_TARGET_DIST, function(dude) return dude:HasTag("tiki") and not dude:HasTag("player") end, MAX_TARGET_SHARES)
    end
end

local function OnNewTarget(inst, data)
	local tar = data.target
	inst.components.combat:ShareTarget(tar, SHARE_TARGET_DIST, function(dude) return dude:HasTag("tiki") and not dude:HasTag("player") end, MAX_TARGET_SHARES)
end

local guardbuilds = {"wanawana_build"}

local function GuardRetargetFn(inst)
    local home = inst.components.homeseeker and inst.components.homeseeker.home
    --defend the king, then the torch, then myself
    local defenseTarget = FindEntity(inst, TUNING.PIG_GUARD_DEFEND_DIST, function(guy) return guy:HasTag("king") end)
    if not defenseTarget and home and inst:GetDistanceSqToInst(home) < SpringMod(TUNING.PIG_GUARD_DEFEND_DIST*TUNING.PIG_GUARD_DEFEND_DIST) then
        defenseTarget = home
    end
    if not defenseTarget then
        defenseTarget = inst
    end
    local invader = FindEntity(defenseTarget or inst, SpringMod(TUNING.PIG_GUARD_TARGET_DIST), function(guy)
        return guy:HasTag("character") and not guy:HasTag("tiki")
    end)
    if not defenseTarget.happy then
        if invader then
            return invader
        end
        if not GetClock():IsDay() and home and home.components.burnable and home.components.burnable:IsBurning() then
            local lightThief = FindEntity(home, home.components.burnable:GetLargestLightRadius(), function(guy)
                return guy:HasTag("player") and not guy:HasTag("tiki")
                       and guy.LightWatcher:IsInLight()
            end)
            if lightThief then
                return lightThief
            end
        end
    end
    return FindEntity(defenseTarget, SpringMod(TUNING.PIG_GUARD_DEFEND_DIST), function(guy)
        return guy:HasTag("monster")
    end)
end

local function GuardKeepTargetFn(inst, target)
    local home = inst.components.homeseeker and inst.components.homeseeker.home
    if home then
        local defendDist = SpringMod(TUNING.PIG_GUARD_DEFEND_DIST)
        if not GetClock():IsDay() and home.components.burnable and home.components.burnable:IsBurning() then
            defendDist = home.components.burnable:GetLargestLightRadius()
        end
        return home:GetDistanceSqToInst(target) < defendDist*defendDist
               and home:GetDistanceSqToInst(inst) < defendDist*defendDist
    end
    return inst.components.combat:CanTarget(target)     
           and not (target.sg and target.sg:HasStateTag("transform") )
end
local function GuardShouldSleep(inst)
    return false
end
local function GuardShouldWake(inst)
    return true
end
local function SetGuardPig(inst)
    inst:AddTag("guard")
    local brain = require "brains/wanawanabrain"
    inst:SetBrain(brain)
    inst:SetStateGraph("SGpig")
	inst.AnimState:SetBuild(inst.build)
    
    inst.components.sleeper:SetResistance(3)

    inst.components.health:SetMaxHealth(1000)
    inst.components.combat:SetDefaultDamage(50)
    inst.components.combat:SetAttackPeriod(TUNING.PIG_GUARD_ATTACK_PERIOD)
    inst.components.combat:SetKeepTargetFunction(GuardKeepTargetFn)
    inst.components.combat:SetRetargetFunction(1, GuardRetargetFn)
    inst.components.combat:SetTarget(nil)
    inst.components.locomotor.runspeed = TUNING.PIG_RUN_SPEED
    inst.components.locomotor.walkspeed = TUNING.PIG_WALK_SPEED
    
    inst.components.sleeper:SetSleepTest(GuardShouldSleep)
    inst.components.sleeper:SetWakeTest(GuardShouldWake)
    
    inst.components.lootdropper:SetLoot({})
    inst.components.lootdropper:AddRandomLoot("meat",3)
    inst.components.lootdropper:AddRandomLoot("pigskin",1)
    inst.components.lootdropper.numrandomloot = 1

    inst.components.talker:StopIgnoringAll()
    inst.components.follower:SetLeader(nil)
end

local function common()
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	local sound = inst.entity:AddSoundEmitter()
	local shadow = inst.entity:AddDynamicShadow()
	shadow:SetSize( 1.5, .75 )
	trans:SetScale(1.25,1.25,1.25)
    inst.Transform:SetFourFaced()

    inst.entity:AddLightWatcher()
    
    inst:AddComponent("talker")
    inst.components.talker.ontalk = ontalk
    inst.components.talker.fontsize = 35
    inst.components.talker.font = TALKINGFONT
    inst.components.talker.offset = Vector3(0,-400,0)

    MakeCharacterPhysics(inst, 50, .5)
    MakePoisonableCharacter(inst)
    
    inst:AddComponent("locomotor") -- locomotor must be constructed before the stategraph
    inst.components.locomotor.runspeed = TUNING.PIG_RUN_SPEED --5
    inst.components.locomotor.walkspeed = TUNING.PIG_WALK_SPEED --3

    inst:AddTag("character")
    inst:AddTag("tiki")
    inst:AddTag("scarytoprey")
    anim:SetBank("pigman")
    anim:PlayAnimation("idle_loop")
    anim:Hide("hat")
    ------------------------------------------
    inst:AddComponent("eater")
    inst.components.eater:SetOmnivore()
	inst.components.eater:SetCanEatHorrible()
    table.insert(inst.components.eater.foodprefs, "RAW")
    table.insert(inst.components.eater.ablefoods, "RAW")
    inst.components.eater.strongstomach = true -- can eat monster meat!
    inst.components.eater:SetOnEatFn(OnEat)
    ------------------------------------------
    inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "pig_torso"
    MakeMediumBurnableCharacter(inst, "pig_torso")
    ------------------------------------------
    inst:AddComponent("follower")
    inst.components.follower.maxfollowtime = TUNING.PIG_LOYALTY_MAXTIME
    ------------------------------------------
    inst:AddComponent("health")
    ------------------------------------------
    inst:AddComponent("inventory")   
    ------------------------------------------
    inst:AddComponent("lootdropper")
    ------------------------------------------
    inst:AddComponent("knownlocations")
    ------------------------------------------
    inst:AddComponent("sleeper")
    ------------------------------------------
    MakeMediumFreezableCharacter(inst, "pig_torso")
    ------------------------------------------
	inst:AddComponent("inspectable")
    ------------------------------------------
    inst.OnSave = function(inst, data)
        data.build = inst.build
    end        
    inst.OnLoad = function(inst, data)    
		if data then
			inst.build = data.build or builds[1]
			inst.AnimState:SetBuild(inst.build)
		end
    end           
    inst:ListenForEvent("attacked", OnAttacked)    
    inst:ListenForEvent("newcombattarget", OnNewTarget)
    return inst
end

local function guard()
    local inst = common()
    inst.build = guardbuilds[math.random(#guardbuilds)]
    inst.AnimState:SetBuild(inst.build)
    SetGuardPig(inst)
    return inst
end

return Prefab("common/character/wanawanatiki", guard, assets, prefabs) 
